<?php

// -------------- displaying pages ------------------------------------

	include('../utils.php');


	function antechamberHeader($title)
	// Prints the header common to all pages in the antechamber
	{
		printHeader($title,"Antechamber");
	}


	function antechamberMenu($currentPage)
	// Prints the menu in the "Antechamber" section.
	{
		printMenu("Antechamber",$currentPage,array("log.php","raw.php","csv.php","finalResults.php"),array("Log File","Raw Results","CSV Formatted Results","Final CPS Results"));
	}


// -------------------- File management -------------------------------

	function printContent($path)
	// Prints the content of the file located in $path between div marks.
	{
		echo '<div class="shell">';
		$content = file($path);
		for ($i=0; $i<count($content); $i++)
			echo $content[$i].'<br/>';
		echo '</div>';
	}


	function listContent($dir,$currentPage)
	/* Lists the content of a directory, creates a link to each one of
	 * the files in it (not including '.' and '..') and suggests erasing
	 * them.*/
	{
		// If we asked it to delete part of the content, well, we do
		if (isset($_POST['files']))
		{
			echo '<div class="diagramm"><h3>Deleted files</h3><ul>'."\n";
			for ($i=0; $i<count($_POST['files']); $i++)
			{
				unlink($dir."/".$_POST['files'][$i]);
				echo '<li>"'.$_POST['files'][$i];
				echo '</li>'."\n";
			}
			echo '</div></ul>'."\n";
		}
		// Then we display what remains.
		$content = opendir($dir);
		echo '<form action="'.$currentPage.'" method="post">';
		echo '<ul>'."\n";
		while($entry = @readdir($content))
			if( ($entry != '.')&&($entry != '..') )
			{
				echo '<li>';
				echo '<input type="checkbox" name="files[]" value="'.$entry.'">';
				echo '<a href="'.$dir.'/'.$entry.'"><strong>'.$entry.'</strong></a>';
				echo '</li>'."\n";
			}
		echo '</ul>';
		echo '<input type="submit" name="delete" value="Delete selected files"> (<strong>warning!</strong> This can NOT be undone!)';
		echo '</form>';
		closedir($content);
	}


	function suggestTransformingOne()
	/* Lists the content of the ./raw directory, and asks whether one of
	 * these should be turned into a csv file.
	 */
	{
		echo '<hr/>';
		// If we asked it to transform a file, we do
		if (isset($_POST['transform']))
		{
			// transforming the file
			$jsonFile = file_get_contents('./raw/'.$_POST['transform']);
			$jsonContent = json_decode($jsonFile,true);
			json2csv($jsonContent,'csv/'.$jsonContent["id"].'.txt');
			// warning the user
			echo '<div class="diagramm">'."\n";
			echo '<p>The '.$_POST['transform'].' file was transformed into a CSV formatted one. You can find it <a href="./csv.php">here</a>.</p>'."\n";
			echo '</div>'."\n";
		}
		// Then we display all of the files in a menu.
		$content = opendir("./raw/");
		echo '<h4>Would you like to turn one of these into a CSV file?</h4>'."\n";
		echo '<p>If you do, pick the one you want to transform below, then click on the <tt>transform</tt> button. <strong>Be careful!</strong> If a csv file already exists for this raw one, it will be overridden!</p>'."\n";
		echo '<form action="raw.php" method="post">'."\n";
		echo '<select name="transform">'."\n";
		while($entry = @readdir($content))
			if( ($entry != '.')&&($entry != '..') )
				echo '<option value="'.$entry.'">'.$entry.'</option>'."\n";
		echo '<input type="submit" name="go_transformation" value="Transform it!">';
		echo '</form>';
		closedir($content);
	}

	function append2log($content,$jobid)
	/* appends the content of the $content variable to a log file, as
	 * well as the date and the identifier of the job. The aim is to
	 * keep a track of the number of webhooks received and of their
	 * type.*/
	{
		$log = fopen('./log.txt','a+');
		fputs($log,"\n".date('m/d/Y, H:i:s')." (job ".$jobid.") : ".$content);
		fclose($log);
	}


	function save($content,$path)
	/* prints the value of the $content in the file whose path is $path. */
	{
		$file = fopen($path,'w+');
		fputs($file,$content);
		fclose($file);
	}


	function erase($path)
	// Erase the file at $path: removes all of its content, leaving an empty file.
	{
		save('',$path);
	}

	function suggestErasing($path,$currentPage)
	/* Prints a button suggesting the erasing of the file at $path. If
	the button is pressed, the file IS erased.*/
	{
		if (isset($_POST["delete"]))
			if ($_POST["delete"] == $path)
				erase($path,true);
		echo '<form action="'.$currentPage.'" method="post">';
		echo '  <input type="hidden" name="delete" value="'.$path.'">';
		echo '	<input type="submit" value="Delete"> (<strong>warning!</strong> This can NOT be undone!)';
		echo '</form>';
	}

// ------------------- Webhooks management ----------------------------

	function jobCompleteNotification($address,$job_id)
	/* sends an email to $address telling that the last job submitted
	 * has now all the results it needs */
	{
		// header
		$headers = 'From: "CPS"'."\n";
		$headers .='Reply-To: None'."\n";
		$headers .='Content-Type: text/plain; charset="iso-8859-1"'."\n";
		$headers .='Content-Transfer-Encoding: 8bit';

		// message
		
		$message .= 'signal sent: '.$_POST['signal']."\n"."\n";
		$message = 'CrowdFlower just  sent a webhook spotting the end of the '.$job_id.' job. This means that all the HITs necessary to perform a new iteration have been answered. You just have to launch a new splitsort iteration: call "./cps --iteration <your MySQL password>" on the directory hosting cps.'."\n";
		$message .="\n";
		$message .='This email was sent automatically the '.date('F d Y').' at '.date('H:i:s').', do not try to answer to it. If you have problems running CPS, you can check its online documentation at http://ns204207.ovh.net/videosense/'."\n";
		$message .="\n";
		$message .='CPS is free software. <inserer ici la bonne licence>'."\n";
		$message .="\n"."\n";
		$message .= '--'."\n";
		$message .= 'Crowd Powered Sort'."\n";
		$message .= 'This is an automatically generated mail, do not answer.'."\n";
		$message .= 'http://ns204207.ovh.net/videosense/'."\n";

		// sending mail
		if (mail($address,'CPS: a new iteration can start!',$message,$headers))
		{
			echo '<h1>success !</h1>';
			echo '<h2>mail sent:</h2>';
			echo '<h3>header:</h3>';
			echo '<p>'.$headers.'</p>';
			echo '<h3>message:</h3>';
			echo '<p>'.$message.'</p>';
		}
		else
			echo '<h1>FAIL !</h1>';
	}


	function finishedNotification($address,$resultsFile)
	/* sends an email to $address telling that the last job submitted
	 * has now all the results it needs */
	{
		// header
		$headers = 'From: "CPS"'."\n";
		$headers .='Reply-To: None'."\n";
		$headers .='MIME-Version: 1.0'."\n";
		$headers .= 'Content-Type: multipart/mixed; boundary="==mimepart0001=="'."\n";
		$headers .= 'Content-Transfer-Encoding: 7bit'."\n";

		// message
		$message = "\n".'--==mimepart0001=='."\n";
		$message .= 'Content-Type: text/plain; charset="iso-8859-1"'."\n";
		$message .= 'Content-Transfer-Encoding: 7bit'."\n";
		$message .= 'The Splitsort is over!'."\n"."\n";
		$message .= 'CPS has finished the sorting of the media, the results are in the attachment. For more information on how this file is formatted, please visit http://ns204207.ovh.net/videosense/userManual/xmlFiles.php (bottom of the page). You can also download it here: http://ns204207.ovh.net/videosense/webhook/finalResults.php'."\n";
		$message .="\n";
		$message .='This email was sent automatically the '.date('F d Y').' at '.date('H:i:s').', do not try to answer to it. If you have problems running CPS, you can check its online documentation at http://ns204207.ovh.net/videosense/'."\n";
		$message .="\n";
		$message .='CPS is free software. <inserer ici la bonne licence>'."\n";
		$message .="\n"."\n";
		$message .= '--'."\n";
		$message .= 'Crowd Powered Sort'."\n";
		$message .= 'This is an automatically generated mail, do not answer.'."\n";
		$message .= 'http://ns204207.ovh.net/videosense/'."\n";

		// attachment
		$message .= '--==mimepart0001=='."\n";
		$message .= 'Content-Type: text/xml; charset="iso-8859-1"; name="CPSresults.xml"
Content-Transfer-Encoding: 7bit
Content-Disposition: attachment; filename="CPSresults.xml"'."\n"."\n";
		$message .= $resultsFile."\n";

		// end of the message
		$message .= "\n".'--==mimepart0001==--';

		// sending mail
		if (mail($address,'CPS: it\'s over!',$message,$headers))
		{
			echo '<h1>success !</h1>';
			echo '<h2>mail sent:</h2>';
			echo '<h3>header:</h3>';
			echo '<p>'.$headers.'</p>';
			echo '<h3>message:</h3>';
			echo '<p>'.$message.'</p>';
		}
		else
			echo '<h1>FAIL !</h1>';
	}


	function json2csv($jsonContent,$path)
	/* Transforms the JSON object $jsonContent into a csv fomatted one
	 * and prints it in the file whose path is $path. */
	{
		$res = '_unit_id,_created_at,_started_at,_id,_trust,_worker_id,_country,_region,_city,_ip,which_media_conveys_the_most,idMedia1,idMedia2,axis,urlmedia1,urlmedia2'."\n";
		$j = $jsonContent["results"]["judgments"];
		$count = count($j);
		for($i=0;$i<$count;$i++)
		{
			// initialisation
			$line = '';
			// building line
			$line .= $j[$i]["unit_id"].',';
			$line .= $j[$i]["created_at"].',';
			$line .= $j[$i]["started_at"].',';
			$line .= $j[$i]["id"].',';
			$line .= $j[$i]["which_media_conveys_the_most"]["confidence"].',';
			$line .= $j[$i]["worker_id"].',';
			$line .= $j[$i]["country"].',';
			$line .= $j[$i]["region"].',';
			$line .= $j[$i]["city"].',';
			$line .= $j[$i]["ip"].',';
			$line .= $j[$i]["judgment"].',';
			// adding the content of the 'data' field:
			$line .= $j[$i]["data"]["idmedia1"].',';
			$line .= $j[$i]["data"]["idmedia2"].',';
			$line .= $j[$i]["data"]["axis"].',';
			$line .= $j[$i]["data"]["urlmedia1"].',';
			$line .= $j[$i]["data"]["urlmedia2"];
//			$line .= $j[$i]["job_id"].',';
//			$line .= $j[$i]["golden"].',';
//			$line .= $j[$i]["unit_state;
//			$line .= $j[$i]["webhook_sent_at"].',';
//			$line .= $j[$i]["reviewed"].',';
//			$line .= $j[$i]["missed"].',';
//			$line .= $j[$i]["tainted"].',';
//			$line .= $j[$i]["external_type"].',';
//			$line .= $j[$i]["rejected"].',';
			// adding the line to the results
			$res .= $line."\n";
//			echo $line."\n";
		}
		save($res,$path);
	}




// I hope the persons responsible for the development of PHP will burn in malebolge for eternity.
?>
