#!/usr/bin/env python
# -*- coding: UTF-8 -*-

""" This stub emulates the behaviour of CrowdFlower's crowd : it takes
the csv file containing media to submit to the turkers and then POSTs
a webhook containing randomly generated results at a given address so
as to imitate CrowdFlower's behaviour as closely as possible.
These results are "obtained" using nTurkers turkers ; each HIT
submitted receives nAnnotation annotations. Each turker has an errRate
error rate to simulate incorrect results. """

# classical ones:
import sys
import random
# mines:
from turker import Turker
from csvReader import CSVreader
from mediaReader import MediaReader
from resultsUploader import ResultsUploader
from order import Order




def errorDistribution(numTurker):
	""" returns the error rate corresponding to a number in
	range(0,nTurker). The aim is to allow to test several distributions
	of error rates among the turkers."""
	x = float(numTurker) / N_TURKERS
	if (x < 0.80):
		return 0.05
	elif (x < 0.9):
		return 0.1
	else:
		return 0.2


def jobEmulation(jobid):
	"""Performs a complete emulation of CrowdFlower:
		- reads the content of a csv file containing the comparisons
		- answers N_ANNOTATIONS times to each question, with a given
			error rate different for each turker
		- posts a "webhook" at a given address containing a signal =
			'job_complete' and a payload = $(results)"""
	# turkers
	tk = []
	for i in range(0,N_TURKERS):
		oneTk = Turker(errorDistribution(i))
		tk.append(oneTk)
	# comparisons
	cr = CSVreader("../data/HITdata/" + str(jobid) + ".csv")
	compNumber = cr.numberOfComparisons()
	results = ResultsUploader(jobid)
	log = open("../data/CM-stub/log.py","w")
	for compId in range(0,compNumber):
		# the following loop is to prevent a turker to answer twice on
		# the same HIT
		rank = [ random.randint(0,N_TURKERS-1) ]
		alea = rank[0]
		for annot in range(1,N_ANNOTATIONS):
			while (alea == rank[annot - 1]): # we keep searching for another turker
				alea = random.randint(0,N_TURKERS-1)
			rank.append(alea)
		for annot in range(0,N_ANNOTATIONS):
			judg = {"data": cr.getComparisonData(compId)}
			judg["created_at"] = "2011-06-23T12:59:51+00:00"
			judg["started_at"] = "2011-06-23T21:37:42+11:11"
			judg["unit_id"] = compId
			judg["country"] = "Tadjikistan"
			judg["city"] = "Douchanbe"
			judg["trust"] = 0.9
			judg["ip"] = str(rank[annot] % 256) + "." + str(256 - rank[annot] % 256) + ".2.3"
			involvedTurker = tk[rank[annot]]
			judg["id"] = compId * 1000000 + annot
			judg["worker_id"] = involvedTurker.id
			# asking a turker to compare the numbers accordingly to the
			# content of 'od'
			great = int(judg["data"]["idmedia1"])
			small = int(judg["data"]["idmedia2"])
			correct = involvedTurker.correct()
			log.write(judg["data"]["axis"] + " " + str(great) + " >? " +
				str(small) + " " + str(correct) + " :  ")
			if (great == od.getGreatest(great,small,judg["data"]["axis"]) ):
				if correct:
					judg["judgment"] = "Media 1"
					log.write("Media 1\n")
				else:
					judg["judgment"] = "Media 2"
					log.write("Media 2\n")
			else:
				if correct:
					judg["judgment"] = "Media 2"
					log.write("Media 2\n")
				else:
					judg["judgment"] = "Media 1"
					log.write("Media 1\n")
			results.addJudgement(judg)
	log.close()
	results.printJSON("../data/CM-stub/json/" + str(jobid) + ".json")
	results.uploadFile("../data/CM-stub/json/" + str(jobid) + ".json",
		URL_ANTECHAMBER)


# ========================== MAIN FUNCTION ===========================


if (__name__ == "__main__"):

	#constants
	N_TURKERS = 20
	N_ANNOTATIONS = 3
	URL_ANTECHAMBER = "http://ns204207.ovh.net/videosense/antechamber/main.php"

	# not enough arguments (this is to prevent an involuntary "click launch")
	if (len(sys.argv) <= 1):
		print "need more arguments, MOAR !!1!\n"
	# acting like a stub
	else:
		mr = MediaReader()
		media = mr.getIds()
		od = Order(media,mr.getAxes())
		if (sys.argv[1] == "check"):
			print "results to check:\n"
			res = input()
			od.checkResults(res)
		else:
			jobEmulation(sys.argv[1])

